package com.example.gpu_video_plugin

import android.app.Activity
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.opengl.GLException
import android.os.Environment
import android.os.Handler
import android.provider.MediaStore
import android.util.Log
import android.view.MotionEvent
import android.view.View
import android.widget.FrameLayout
import androidx.annotation.NonNull
import com.example.gpu_video_plugin.gpuv.camerarecorder.CameraRecordListener
import com.example.gpu_video_plugin.gpuv.camerarecorder.GPUCameraRecorder
import com.example.gpu_video_plugin.gpuv.camerarecorder.GPUCameraRecorderBuilder
import com.example.gpu_video_plugin.gpuv.camerarecorder.LensFacing
import com.example.gpu_video_plugin.widget.SampleCameraGLView
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.nio.IntBuffer
import java.text.SimpleDateFormat
import java.util.*
import javax.microedition.khronos.egl.EGL10
import javax.microedition.khronos.egl.EGLContext
import javax.microedition.khronos.opengles.GL10

class Camera(context1: Context, activity1: Activity) {
    private var context: Context = context1
    private var activity: Activity = activity1
    private var sampleGLView: SampleCameraGLView? = null
    private var GPUCameraRecorder: GPUCameraRecorder? = null
    private var frameLayout: FrameLayout? = null
    private var cameraHeight: Int = 0
    private var cameraWidth: Int = 0
    private var lensFacing = LensFacing.BACK
    private lateinit var filepath: String
    private fun createBitmapFromGLSurface(w: Int, h: Int, gl: GL10): Bitmap? {
        val bitmapBuffer = IntArray(w * h)
        val bitmapSource = IntArray(w * h)
        val intBuffer = IntBuffer.wrap(bitmapBuffer)
        intBuffer.position(0)
        try {
            gl.glReadPixels(0, 0, w, h, GL10.GL_RGBA, GL10.GL_UNSIGNED_BYTE, intBuffer)
            var offset1: Int
            var offset2: Int
            var texturePixel: Int
            var blue: Int
            var red: Int
            var pixel: Int
            for (i in 0 until h) {
                offset1 = i * w
                offset2 = (h - i - 1) * w
                for (j in 0 until w) {
                    texturePixel = bitmapBuffer[offset1 + j]
                    blue = texturePixel shr 16 and 0xff
                    red = texturePixel shl 16 and 0x00ff0000
                    pixel = texturePixel and -0xff0100 or red or blue
                    bitmapSource[offset2 + j] = pixel
                }
            }
        } catch (e: GLException) {
            Log.e("CreateBitmap", "createBitmapFromGLSurface: " + e.message, e)
            return null
        }
        return Bitmap.createBitmap(bitmapSource, w, h, Bitmap.Config.ARGB_8888)
    }

    private interface BitmapReadyCallbacks {
        fun onBitmapReady(bitmap: Bitmap?)
    }

    private fun captureBitmap(bitmapReadyCallbacks: BitmapReadyCallbacks) {
        sampleGLView!!.queueEvent {
            val egl = EGLContext.getEGL() as EGL10
            val gl = egl.eglGetCurrentContext().gl as GL10
            val snapshotBitmap: Bitmap? = createBitmapFromGLSurface(sampleGLView!!.measuredWidth, sampleGLView!!.measuredHeight, gl)
            activity.runOnUiThread(Runnable { bitmapReadyCallbacks.onBitmapReady(snapshotBitmap) })
        }
    }

    fun saveAsPngImage(bitmap: Bitmap, filePath: String?) {
        try {
            val file = File(filePath)
            val outStream = FileOutputStream(file)
            bitmap.compress(Bitmap.CompressFormat.PNG, 100, outStream)
            outStream.close()
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }


    fun exportMp4ToGallery(context: Context, filePath: String) {
        val values = ContentValues(2)
        values.put(MediaStore.Video.Media.MIME_TYPE, "video/mp4")
        values.put(MediaStore.Video.Media.DATA, filePath)
        context.contentResolver.insert(MediaStore.Video.Media.EXTERNAL_CONTENT_URI,
                values)
        context.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE,
                Uri.parse("file://$filePath")))
    }

    private fun getVideoFilePath(): String {
        return getAndroidMoviesFolder().absolutePath + "/" + SimpleDateFormat("yyyyMM_dd-HHmmss").format(Date()) + "GPUCameraRecorder.mp4"
    }

    private fun getAndroidMoviesFolder(): File {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MOVIES)
    }

    private fun exportPngToGallery(context: Context, filePath: String) {
        val mediaScanIntent = Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE)
        val f = File(filePath)
        val contentUri = Uri.fromFile(f)
        mediaScanIntent.data = contentUri
        context.sendBroadcast(mediaScanIntent)
    }

    fun getImageFilePath(): String {
        return getAndroidImageFolder().absolutePath + "/" + SimpleDateFormat("yyyyMM_dd-HHmmss").format(Date()) + "GPUCameraRecorder.png"
    }

    private fun getAndroidImageFolder(): File {
        return Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES)
    }

    fun imageCapture() {
        captureBitmap(object : BitmapReadyCallbacks {
            override fun onBitmapReady(bitmap: Bitmap?) {
                Handler().post {
                    val imagePath = getImageFilePath()
                    saveAsPngImage(bitmap!!, imagePath)
                    exportPngToGallery(context, imagePath)
                }
            }
        })
    }

    fun flash() {
        if (GPUCameraRecorder != null && GPUCameraRecorder!!.isFlashSupport) {
            GPUCameraRecorder!!.switchFlashMode()
            GPUCameraRecorder!!.changeAutoFocus()
        }
    }

    //    private void changeFilter(Filters filters) {
    //        GPUCameraRecorder.setFilter(Filters.getFilterInstance(filters, getApplicationContext()));
    //    }

    fun switchCamera() {

//        releaseCamera()
        lensFacing = if (lensFacing == LensFacing.BACK) {
            LensFacing.FRONT
        } else {
            LensFacing.BACK
        }
        if (sampleGLView != null) {
            sampleGLView!!.onPause()
        }
        if (GPUCameraRecorder != null) {
            GPUCameraRecorder!!.stop()
            GPUCameraRecorder!!.release()
            GPUCameraRecorder = null
        }
        if (sampleGLView != null) {
//            frameLayout?.removeView(sampleGLView)
            sampleGLView = null
        }
//        sampleGLView = SampleCameraGLView(context)
        activity.runOnUiThread(Runnable {
//            frameLayout = FrameLayout(context)
//            frameLayout!!.removeAllViews()
            sampleGLView = null
            sampleGLView = SampleCameraGLView(context)
            sampleGLView!!.setTouchListener { event: MotionEvent, width: Int, height: Int ->
                if (GPUCameraRecorder == null) return@setTouchListener
                GPUCameraRecorder!!.changeManualFocusPoint(event.x, event.y, width, height)
            }
            GPUCameraRecorder = GPUCameraRecorderBuilder(activity, sampleGLView) //.recordNoFilter(true)
                    .cameraRecordListener(object : CameraRecordListener {
                        override fun onGetFlashSupport(flashSupport: Boolean) {
                            activity.runOnUiThread(Runnable {
//                            findViewById<View>(R.id.btn_flash).setEnabled(flashSupport)
                            })
                        }

                        override fun onRecordComplete() {
                            exportMp4ToGallery(context, filepath)
                        }

                        override fun onRecordStart() {
                            activity.runOnUiThread(Runnable {
//                            lv.setVisibility(View.GONE)
                            })
                        }

                        override fun onError(exception: Exception) {
                            Log.e("GPUCameraRecorder", exception.toString())
                        }

                        override fun onCameraThreadFinish() {
//                        if (toggleClick) {
//                            activity.runOnUiThread(Runnable {
//                                setUpCamera(call, result)
//                            })
//                        }
//                        toggleClick = false;
                        }

                        override fun onVideoFileReady() {}
                    })
                    .videoSize(cameraWidth, cameraHeight)
                    .cameraSize(cameraWidth, cameraHeight)
                    .lensFacing(lensFacing)
                    .build()
            frameLayout!!.addView(sampleGLView)
        })


//        toggleClick = true
    }

    fun setFilter(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        val nameGlFilter: String = call.arguments as String
        val filterType: FilterType = getFilter(nameGlFilter)
        if (GPUCameraRecorder != null) {
            GPUCameraRecorder!!.setFilter(FilterType.createGlFilter(filterType, context))
        }
        result.success("ok")
    }

    private fun getFilter(nameFilter: String): FilterType {
        val filters = FilterType.values()
        var filterType: FilterType = filters[2]
        for (i in filters) {
            if (i.name == nameFilter) {
                filterType = i
            }
        }
        return filterType
    }

    fun releaseCamera() {
        Log.d("BBB", "releaseCamera")
        if (sampleGLView != null) {
            sampleGLView!!.onPause()
        }
        if (GPUCameraRecorder != null) {
            GPUCameraRecorder!!.stop()
            GPUCameraRecorder!!.release()
            GPUCameraRecorder = null
        }
        if (sampleGLView != null) {
            frameLayout?.removeView(sampleGLView)
            sampleGLView = null
        }
    }

    fun startRecord() {
        filepath = getVideoFilePath()
        GPUCameraRecorder!!.start(filepath)
    }

    fun stopRecord() {
        GPUCameraRecorder!!.stop()
    }


    private fun setUpCameraView() {
        activity.runOnUiThread(Runnable {
            frameLayout = FrameLayout(context)
            frameLayout!!.removeAllViews()
            sampleGLView = null
            sampleGLView = SampleCameraGLView(context)
            sampleGLView!!.setTouchListener { event: MotionEvent, width: Int, height: Int ->
                if (GPUCameraRecorder == null) return@setTouchListener
                GPUCameraRecorder!!.changeManualFocusPoint(event.x, event.y, width, height)
            }
            frameLayout!!.addView(sampleGLView)
        })
    }

    fun setUpCamera(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result): FrameLayout? {
        val data: HashMap<String, Any?> = call.arguments as HashMap<String, Any?>
        cameraHeight = data["cameraHeight"] as Int
        cameraWidth = data["cameraWidth"] as Int
        sampleGLView = null
        sampleGLView = SampleCameraGLView(context)
//        camera = Camera(context, activity, sampleGLView!!, GPUCameraRecorder)
        setUpCameraView()

////        result.success(idCamera)
//        Log.d("BBB","idCamera:"+idCamera)
//        idCamera++


        GPUCameraRecorder = GPUCameraRecorderBuilder(activity, sampleGLView) //.recordNoFilter(true)
                .cameraRecordListener(object : CameraRecordListener {
                    override fun onGetFlashSupport(flashSupport: Boolean) {
                        activity.runOnUiThread(Runnable {
//                            findViewById<View>(R.id.btn_flash).setEnabled(flashSupport)
                        })
                    }

                    override fun onRecordComplete() {
                        exportMp4ToGallery(context, filepath)
                    }

                    override fun onRecordStart() {
                        activity.runOnUiThread(Runnable {
//                            lv.setVisibility(View.GONE)
                        })
                    }

                    override fun onError(exception: Exception) {
                        Log.e("GPUCameraRecorder", exception.toString())
                    }

                    override fun onCameraThreadFinish() {
//                        if (toggleClick) {
//                            activity.runOnUiThread(Runnable {
//                                setUpCamera(call, result)
//                            })
//                        }
//                        toggleClick = false;
                    }

                    override fun onVideoFileReady() {}
                })

                .videoSize(cameraWidth, cameraHeight)
                .cameraSize(cameraWidth, cameraHeight)
                .lensFacing(lensFacing)
                .build()
        return frameLayout


    }

    companion object {
        const val PERMISSION_REQUEST_CODE = 99999
    }
}