package com.example.gpu_video_plugin

import android.Manifest
import android.app.Activity
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.graphics.Bitmap
import android.net.Uri
import android.opengl.GLException
import android.os.Build
import android.os.Environment
import android.os.Handler
import android.provider.MediaStore
import android.util.Log
import android.view.MotionEvent
import android.widget.FrameLayout
import android.widget.Toast
import androidx.annotation.NonNull
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.example.gpu_video_plugin.gpuv.camerarecorder.CameraRecordListener
import com.example.gpu_video_plugin.gpuv.camerarecorder.GPUCameraRecorder
import com.example.gpu_video_plugin.gpuv.camerarecorder.GPUCameraRecorderBuilder
import com.example.gpu_video_plugin.gpuv.camerarecorder.LensFacing
import com.example.gpu_video_plugin.widget.*
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import java.io.File
import java.io.FileNotFoundException
import java.io.FileOutputStream
import java.io.IOException
import java.nio.IntBuffer
import java.text.SimpleDateFormat
import java.util.*
import javax.microedition.khronos.egl.EGL10
import javax.microedition.khronos.egl.EGLContext
import javax.microedition.khronos.opengles.GL10


/** GpuVideoPlugin */
class GpuVideoPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var context: Context
    private lateinit var activity: Activity
    private lateinit var mp4Compose: Mp4Compose
    private lateinit var call1: MethodCall
    private lateinit var result1: Result
    private var camera: Camera? = null
    private lateinit var flutterBinding: FlutterPlugin.FlutterPluginBinding

    //    private var image_video_id = 0
    private var movieViewId = 0
    private var cameraId = 0

    //    private var movieWrapperView: MovieWrapperView? = null
    private var moviePreview: MoviePreview? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        flutterBinding = flutterPluginBinding
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "gpu_video_plugin")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
        flutterPluginBinding
                .platformViewRegistry
                .registerViewFactory("image_video_file", ImageVideoFactory())


    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "check_permissions_WRITE_EXTERNAL_STORAGE") {
            if (checkPermission_WRITE_EXTERNAL_STORAGE()) result.success(true)
            else result.success(false)
        } else if (call.method == "videoLoaders") {
            if (checkPermission_WRITE_EXTERNAL_STORAGE()) mp4Compose.videoLoaders(call, result);
            else {
                call1 = call
                result1 = result
            }
        }
//        else if (call.method == "get_id_image") {
//            image_video_id++
////            flutterBinding
////                    .platformViewRegistry
////                    .registerViewFactory("image_video_file_$image_video_id", ImageVideoFactory())
//            result.success(image_video_id)
//        }
        else if (call.method == "filters") {
            filters(call, result)
        } else if (call.method == "startCodec") {
            mp4Compose.startCodec(call, result, channel)
        } else if (call.method == "playMovie") {
            mp4Compose.playMovie(call, result)
        }
        else if (call.method == "checkPermission_CAMERA") {
            if (checkPermission_CAMERA()) {
                result.success(true)
            }
            else{
                call1 = call
                result1 = result
            }
        }
        else if (call.method == "setUpCamera") {
            Log.d("BBB", "setUpCamera")
            if (checkPermission_CAMERA()) {
                cameraId++
                if (camera == null) {
                    camera = Camera(context, activity)
                }
                val frameLayout = camera?.setUpCamera(call, result)
                if (frameLayout != null) {
                    flutterBinding
                            .platformViewRegistry
                            .registerViewFactory("gpu_video_plugin_camera_$cameraId", CameraFactory(frameLayout))
                }
                Log.d("BBB", "cameraId:$cameraId")
                result.success(cameraId)
            } else {
                call1 = call
                result1 = result
            }
        } else if (call.method == "get_id_camera") {
            cameraId++
            result.success(cameraId)
        } else if (call.method == "releaseCamera") {
            camera?.releaseCamera()
            result.success("ok")
        } else if (call.method == "setFilter") {
            camera?.setFilter(call, result)
        } else if (call.method == "switchCamera") {
            camera?.switchCamera()
//            cameraId++
//            val frameLayout = camera?.setUpCamera(call, result)
//            if (frameLayout != null) {
//                flutterBinding
//                        .platformViewRegistry
//                        .registerViewFactory("gpu_video_plugin_camera_$cameraId", CameraFactory(frameLayout))
//            }
            result.success("ok")
        } else if (call.method == "flash") {
            camera?.flash()
            result.success("ok")
        } else if (call.method == "imageCapture") {
            camera?.imageCapture()
            result.success("ok")
        } else if (call.method == "startRecord") {
            camera?.startRecord()
            result.success("ok")
        } else if (call.method == "stopRecord") {
            camera?.stopRecord()
            result.success("ok")
        } else if (call.method == "set_up_movie_view") {
            movieViewId++
            moviePreview = null
            moviePreview = MoviePreview(context)
            val movieWrapperView = moviePreview?.setUpMovieView(call, result, channel)
            if (movieWrapperView != null) {
                flutterBinding
                        .platformViewRegistry
                        .registerViewFactory("movie_view_$movieViewId", MovieViewFactory(movieWrapperView))
            }
            result.success(movieViewId)

        } else if (call.method == "remove_view_movie") {
            moviePreview?.removeView()
            result.success("ok")
        } else if (call.method == "set_filter_movie") {
            moviePreview?.setFilterMovie(call, result)
        } else if (call.method == "seekToMovie") {
            moviePreview?.seekToMovie(call, result)
        } else if (call.method == "changeValueFilter") {
            moviePreview?.changeValueFilter(call, result)
        } else if (call.method == "setPlay") {
            moviePreview?.setPlay(call, result)
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
        mp4Compose = Mp4Compose(context, activity)

        binding.addRequestPermissionsResultListener(object : PluginRegistry.RequestPermissionsResultListener {
            override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>?, grantResults: IntArray?): Boolean {
                Log.d("BBB", "addRequestPermissionsResultListener")
                when (requestCode) {
                    Mp4Compose.PERMISSION_REQUEST_CODE -> if (grantResults!!.size > 0 && grantResults!![0] == PackageManager.PERMISSION_GRANTED) {
                        Toast.makeText(context, "permission has been grunted.", Toast.LENGTH_SHORT).show()
                        if (call1.method == "videoLoaders") {
                            mp4Compose.videoLoaders(call1, result1)
                        }
//                        channel.invokeMethod("requested_WRITE_EXTERNAL_STORAGE", true)

                    } else {
                        Toast.makeText(context, "[WARN] permission is not grunted.", Toast.LENGTH_SHORT).show()
//                        channel.invokeMethod("requested_WRITE_EXTERNAL_STORAGE", false)
                        result1.notImplemented()
                    }
                    Camera.PERMISSION_REQUEST_CODE -> if (grantResults!!.size > 0 && grantResults!![0] == PackageManager.PERMISSION_GRANTED) {
                        Toast.makeText(context, "permission has been grunted.", Toast.LENGTH_SHORT).show()
                        if (call1.method == "checkPermission_CAMERA") {
                            result1.success(true)
                        }
//                        channel.invokeMethod("requested_WRITE_EXTERNAL_STORAGE", true)

                    } else {
                        Toast.makeText(context, "[WARN] permission is not grunted.", Toast.LENGTH_SHORT).show()
//                        channel.invokeMethod("requested_WRITE_EXTERNAL_STORAGE", false)
                        result1.success(false)
                    }
                }
                return false;
            }

        })

    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivity() {
    }


    private fun filters(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        val filters = FilterType.values()
        val list: ArrayList<String> = ArrayList()
        for (i in filters) {
            list.add(i.name)
        }
        result.success(list)
    }


    private fun checkPermission_WRITE_EXTERNAL_STORAGE(): Boolean {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            return true
        }
        // request permission if it has not been grunted.
        if (ContextCompat.checkSelfPermission(context, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(activity, arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE), Mp4Compose.PERMISSION_REQUEST_CODE)
            return false
        }
        return true
    }

    private fun checkPermission_CAMERA(): Boolean {
        Log.d("BBB", "checkPermission_CAMERA")
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            return true
        }
        // request permission if it has not been grunted.
        if (ContextCompat.checkSelfPermission(context, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED ||
                ContextCompat.checkSelfPermission(context, Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(activity, arrayOf(Manifest.permission.CAMERA, Manifest.permission.RECORD_AUDIO), Camera.PERMISSION_REQUEST_CODE)
            return false
        }
        return true
    }
}
