package com.example.gpu_video_plugin

import android.content.Context
import android.net.Uri
import android.util.Log
import android.view.ViewGroup
import android.widget.RelativeLayout
import androidx.annotation.NonNull
import com.example.gpu_video_plugin.gpuv.egl.filter.GlFilter
import com.example.gpu_video_plugin.gpuv.player.GPUPlayerView
import com.example.gpu_video_plugin.widget.MovieWrapperView
import com.example.gpu_video_plugin.widget.PlayerTimer
import com.google.android.exoplayer2.MediaItem
import com.google.android.exoplayer2.SimpleExoPlayer
import com.google.android.exoplayer2.trackselection.DefaultTrackSelector
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.util.*

class MoviePreview(context1: Context) {
    private var context: Context = context1
    private var player: SimpleExoPlayer? = null
    private var gpuPlayerView: GPUPlayerView? = null
    private var playerTimer: PlayerTimer? = null
    private lateinit var movieWrapperView: MovieWrapperView
    private var filter: GlFilter? = null
    private var adjuster: FilterAdjuster? = null

    fun setPlay(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        val isPlay: Boolean = call.arguments as Boolean
        if (player == null) return
        player!!.playWhenReady = isPlay
        result.success("ok")
    }

    fun seekToMovie(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        val progress: Int = call.arguments as Int
        if (player == null) return
        player!!.seekTo((progress).toLong())
        result.success("ok")
    }

    fun changeValueFilter(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        val percentage: Int = call.arguments as Int
        if (adjuster != null) {
            adjuster!!.adjust(filter, percentage)
            result.success("ok")
        } else {
            result.error("ERROR", "Filter not change value", null)
        }

    }

    fun setFilterMovie(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
//        val data: HashMap<String, Any?> = call.arguments as HashMap<String, Any?>
        val nameGlFilter: String = call.arguments as String
        val filters = FilterType.values()
        var filterType: FilterType = filters[2]
        for (i in filters) {
            if (i.name == nameGlFilter) {
                filterType = i
            }
        }
        filter = FilterType.createGlFilter(filterType, context)
        adjuster = FilterType.createFilterAdjuster(filterType)
//        findViewById<View>(R.id.filterSeekBarLayout).setVisibility(if (adjuster != null) View.VISIBLE else View.GONE)
        gpuPlayerView!!.setGlFilter(filter)
        result.success(adjuster != null)
    }


    fun setUpMovieView(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result, methodChannel: MethodChannel): MovieWrapperView {
        val url: String = call.arguments as String
        setUpSimpleExoPlayer(url)
        setUoGlPlayerView()
        setUpTimer(methodChannel)
        return movieWrapperView
    }

    private fun setUpSimpleExoPlayer(url: String) {
        // SimpleExoPlayer
        player = SimpleExoPlayer.Builder(context)
                .setTrackSelector(DefaultTrackSelector(context))
                .build()
        player?.addMediaItem(MediaItem.fromUri(Uri.parse(url)))
        player?.prepare()
        player?.playWhenReady = true
    }

    private fun setUoGlPlayerView() {
        gpuPlayerView = GPUPlayerView(context)
        gpuPlayerView?.setSimpleExoPlayer(player)
        gpuPlayerView?.layoutParams = RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
        movieWrapperView = MovieWrapperView(context)
        movieWrapperView.addView(gpuPlayerView)
        gpuPlayerView?.onResume()
    }

    private fun setUpTimer(methodChannel: MethodChannel) {
        playerTimer = PlayerTimer()
        playerTimer?.setCallback(PlayerTimer.Callback {
            var position = player?.currentPosition?.toInt()
            val duration = player?.duration?.toInt()
            if (duration != null) {
                if (duration <= 0) return@Callback
            }
            if (position != null && duration != null) {
                if (position > duration) position = duration;
                val mutableMap: MutableMap<String, Any> = mutableMapOf<String, Any>()
                mutableMap["duration"] = duration
                mutableMap["currentPosition"] = position
                methodChannel.invokeMethod("time_movie_view", mutableMap)
//                if(position==duration)return@Callback
                Log.d("BBB", "time:" + position);
            }

//            timeSeekBar.setMax(duration.toInt() / 1000)
//            timeSeekBar.setProgress(position.toInt() / 1000)
        })
        playerTimer?.start()
    }

    fun removeView() {
        Log.d("BBB", "removeView: MoviePreview")
        releasePlayer()
        if (playerTimer != null) {
            playerTimer?.stop()
            playerTimer?.removeMessages(0)
        }
    }

    private fun releasePlayer() {
        gpuPlayerView?.onPause()
        movieWrapperView.removeAllViews()
        gpuPlayerView = null
        player?.stop()
        player?.release()
        player = null
    }
}