import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class ProcessMovieView extends StatefulWidget {
  const ProcessMovieView(
      {Key? key,
      required this.onDuration,
      required this.onCurrentPosition,
      required this.child})
      : super(key: key);
  final ValueSetter<int> onCurrentPosition;
  final ValueSetter<int> onDuration;
  final Widget child;

  @override
  _ProcessMovieViewState createState() => _ProcessMovieViewState();
}

class _ProcessMovieViewState extends State<ProcessMovieView> {
  final MethodChannel _channel = const MethodChannel('gpu_video_plugin');

  Future<void> _fromNative(MethodCall call) async {
    if (call.method == 'time_movie_view') {
      final maps = call.arguments;
      print('callTest result1 = ${maps}');
      var duration = maps["duration"] as int;
      widget.onDuration(duration);
      var currentPosition = maps["currentPosition"] as int;

      widget.onCurrentPosition(currentPosition);

      setState(() {});
    }
  }

  @override
  void initState() {
    super.initState();
    _channel.setMethodCallHandler(_fromNative);
  }

  @override
  Widget build(BuildContext context) {
    return widget.child;
  }
}

class MovieView extends StatefulWidget {
  const MovieView({Key? key, required this.url, this.width, this.height})
      : super(key: key);
  static const MethodChannel channel = MethodChannel('gpu_video_plugin');
  final String url;
  final double? height;
  final double? width;

  static Future<bool> setFilterMovie(String nameFilter) async {
    return await channel.invokeMethod("set_filter_movie", nameFilter) as bool;
  }

  static seekToMovie(int progress) {
    channel.invokeMethod("seekToMovie", progress);
  }

  static changeValueFilter(int percentage) {
    channel.invokeMethod("changeValueFilter", percentage);
  }

  static play() {
    channel.invokeMethod("setPlay", true);
  }

  static pause() {
    channel.invokeMethod("setPlay", false);
  }

  @override
  _MovieViewState createState() => _MovieViewState();
}

class _MovieViewState extends State<MovieView> {
  int movieViewId = 1;

  setUpView() async {
    movieViewId = await MovieView.channel
        .invokeMethod("set_up_movie_view", widget.url) as int;
  }

  removeView() async {
    await MovieView.channel.invokeMethod("remove_view_movie");
  }

  @override
  void initState() {
    setUpView();
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
    removeView();
  }

  @override
  Widget build(BuildContext context) {
    String viewType = 'movie_view_$movieViewId';
    Map<String, dynamic> creationParams = <String, dynamic>{};
    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: Center(
        child: AndroidView(
          viewType: viewType,
          layoutDirection: TextDirection.ltr,
          creationParams: creationParams,
          creationParamsCodec: const StandardMessageCodec(),
        ),
      ),
    );
  }
}
